@@ -15,7 +15,14 @@ class DaikinAirBase(DaikinBRP069):
         DaikinBRP069.TRANSLATIONS,
         **{
             'mode': {'0': 'fan', '1': 'hot', '2': 'cool', '3': 'auto', '7': 'dry',},
-            'f_rate': {'1': 'low', '3': 'mid', '5': 'high',},
+            'f_rate': {
+                '1': 'low',
+                '3': 'mid',
+                '5': 'high',
+                '1a': 'low/auto',
+                '3a': 'mid/auto',
+                '5a': 'high/auto',
+            },
         },
     )
 
@@ -28,6 +35,15 @@ class DaikinAirBase(DaikinBRP069):
 
     INFO_RESOURCES = DaikinBRP069.INFO_RESOURCES + ['aircon/get_zone_setting']
 
+    @staticmethod
+    def parse_response(response_body):
+        """Parse response from Daikin, add support for f_rate-auto."""
+        _LOGGER.debug("Parsing %s", response_body)
+        response = super().parse_response(response_body)
+        if response.get('f_auto') == '1':
+            response['f_rate'] = f'{response["f_rate"]}a'
+        return response
+
     def __init__(self, device_id, session=None):
         """Init the pydaikin appliance, representing one Daikin AirBase (BRP15B61) device."""
         super().__init__(device_id, session)
@@ -38,8 +54,6 @@ class DaikinAirBase(DaikinBRP069):
         await super().init()
         if not self.values:
             raise Exception("Empty values.")
-        if self.values['frate_steps'] == '2':
-            self.TRANSLATIONS['f_rate'] = {'1': 'low', '5': 'high'}
 
     async def _run_get_resource(self, resource):
         """Make the http request."""
@@ -61,19 +75,29 @@ class DaikinAirBase(DaikinBRP069):
         """Return True if the device is not an AirBase unit."""
         return False
 
+    @property
+    def fan_rate(self):
+        """Return list of supported fan rates."""
+        fan_rates = list(map(str.title, self.TRANSLATIONS.get('f_rate', {}).values()))
+        if self.values.get('frate_steps') == '2':
+            if self.values.get('en_frate_auto') == '0':
+                return fan_rates[:3:2]
+            return fan_rates[:3:2] + fan_rates[3::2]
+        if self.values.get('en_frate_auto') == '0':
+            return fan_rates[:3]
+        return fan_rates
+
     async def set(self, settings):
         """Set settings on Daikin device."""
         await self._update_settings(settings)
 
-        query_c = 'aircon/set_control_info?pow={pow}&mode={mode}&stemp={stemp}&shum={shum}'.format(
-            **self.values
-        )
-        query_c += '&lpw=&f_airside=0'
-
-        # Apparently some remote controllers doesn't support f_rate and f_dir
-        if self.support_fan_rate:
-            query_c += f'&f_rate={self.values["f_rate"]}'
-        query_c += f'&f_dir={self.values["f_dir"]}'
+        self.values['f_auto'] = '1' if 'a' in self.values["f_rate"] else '0'
+        query_c = (
+            'aircon/set_control_info'
+            '?pow={pow}&mode={mode}&stemp={stemp}&shum={shum}'
+            '&f_rate={f_rate[0]}&f_auto={f_auto}&f_dir={f_dir}'
+            '&lpw=&f_airside=0'
+        ).format(**self.values)
 
         _LOGGER.debug("Sending query_c: %s", query_c)
         await self._get_resource(query_c)
