@@ -89,7 +89,7 @@ class DaikinSkyFi(Appliance):
         """Parse response from Daikin and map it to general Daikin format."""
         _LOGGER.debug("Parsing %s", response_body)
         response = dict([e.split('=') for e in response_body.split('&')])
-        if response['fanflags'] == '3':
+        if response.get('fanflags') == '3':
             response['fanspeed'] = str(int(response['fanspeed']) + 4)
         response.update(
             {
